# vmesaverestore.tcl---saving and restoring card setups


#    derived from camac version

set debug 0
set logging 0
set SSRstarted 0
set CSRstarted 0

set frame SAVERESTORE
set frametitle ""
set icontitle SaveRestore

set Configdir "$env(MIDASEXPBASE)"
catch {set Configdir "$env(EXPERIMENTBASE)"}

set modulerange "#\[?,??\]"

set file ""
set savefile ""
set directory ""
set selection {}
set lowmodule 1
set highmodule 1
set card 0
set cardtypes {}
set cardtype  ""

proc set-experiment dir {
   global env Configdir
   set Configdir $dir
}

proc record-current-save-file {w f} {
   global Configdir RegisterServer

   set z [catch {open [file join $Configdir [set RegisterServer]_current_[set w]] w} file]
   if {$z != 0} then {set-footer "$file"; return}
   puts $file "$f"
   close $file
}

proc recover-current-save-file {w} {
   global env Configdir RegisterServer

   set z [catch {open [file join $Configdir [set RegisterServer]_current_[set w]] r} file]
   if {$z != 0} then {set-footer "$file"; return}
   set f [gets $file]
   close $file

#    return in the format for this OS

   switch $env(platform) {
      windows {set f [midas-map-filename fromUnix $f]}
      unix    {set f [midas-map-filename fromWindows $f]}
   }

   return $f
}

proc repaint {} {
   global lowmodule highmodule card directory
   global SSRstarted CSRstarted

   if {$SSRstarted == 1} then {
      :modify LOW       :value $lowmodule
      :modify HIGH      :value $highmodule
      :modify CARD      :value $card
   }
   :modify DIRECTORY :value "$directory"
}

proc click-show {} {
   global directorylist selection directory
   if {$selection == {}} {return}
   midas-show-file [file join $directory [last [lindex $directorylist $selection]]]
}

proc click-directory name {
   global directory
   set directory $name
   click-redisplay
}

proc click-list {v i} {
   global selection 
   set selection $i        
}

proc click-low v {
   global lowmodule highmodule
   set lowmodule $v
   if {$lowmodule<0} then  {set lowmodule 0}
   if {$lowmodule>255} then {set lowmodule 255}
   if {$highmodule<$lowmodule} then {set highmodule $lowmodule}
   repaint
}

proc click-high v {
   global lowmodule highmodule
   set highmodule $v
   if {$highmodule<0} then {set highmodule 0}
   if {$highmodule>255} then {set highmodule 255}
   if {$lowmodule>$highmodule} then {set lowmodule $highmodule}
   repaint
}

proc click-card v {
   global card cardtype cardtypes 
   set card $v
   set cardtype [lindex $cardtypes $v]
}

proc click-redisplay {} {
   global directory selection
   read-directory $directory
   paint-directory $directory
   set selection {}
   repaint
}

proc click-up {} {
  global directory
  set directory [directory-up $directory]
  click-redisplay
}

proc click-down {} {
  global directory selection
  set directory [directory-down $directory $selection]
  click-redisplay
}

proc saveVMEregisters {file} {
   global debug logging RegisterServer EGreport moduledefinition 
   global modulerange

   set EGreport_save $EGreport
   set EGreport -1
   set-busy

   foreach item $moduledefinition {
      set moduletype [first $item]
      set module [second $item]
      if {$moduletype != ""} then {
          if {$debug} then {puts stdout "saving module $moduletype"}
          set-footer "saving module $moduletype"
          if {$logging == 1} then {insert-log "saving module $moduletype"}
          set registersavepatterns [send vmecontrol inquire \{[first $item]_registersavepatterns\}]
          if {$debug} then {puts stdout "saving $registersavepatterns"}
          if {$logging == 1} then {insert-log "saving $registersavepatterns"}
          foreach pattern $registersavepatterns {
             set register /$RegisterServer/[set module][set modulerange].$pattern
#             set-footer "saving parameters $register ..."
              if {$logging == 1} then {insert-log "saving parameters $register ..."}
             set z [catch {EG "register :pattern '$register' ?names-and-values" } list]
             foreach item $list {puts $file [list $item]}
          }
      }
   }
   set EGreport $EGreport_save
   clear-busy
}


proc click-SSRsave {} {
   global directory savefile
   clear-footer
   catch {:file mkdir $directory} zzz
   set savefile [file join $directory latest]
   set z [catch {open $savefile w} file]
   if {$z != 0} then {set-footer $file; return}
   set z [catch {saveVMEregisters $file} m]
   close $file
   click-redisplay
   if {$z!=0} then {set-footer "save failed: $m"; return}
   set-footer "saved current card setup in file 'latest'"
   record-current-save-file setup $savefile
   write-registers NetVar_HasBeenRestored 1
}

proc click-CSRsave {} {
   global directory savefile
   set configuration [send vmecontrol Configuration]
   clear-footer
   catch {:file mkdir $directory} zzz
   set savefile [file join $directory latest]
   set z [catch {open $savefile w} file]
   if {$z != 0} then {set-footer $file; return}
   foreach crate $configuration {
       foreach item $crate {
          puts $file "$item"
       }
   }
   close $file
   click-redisplay
   set-footer "saved current card configuration in file 'latest'"
   record-current-save-file config $savefile
}

proc click-time-stamp {} {
    global directory CSRstarted SSRstarted
    if {![file exists [file join $directory latest]]} then \
	    {set-footer "there's no latest save-set!!!"; return}
    set stamp "[date +%Y%h%d-%H.%M.%S]"
    set savefile [file join $directory $stamp]
    set z [catch {:file rename [file join $directory latest] $savefile} m]
    if {$z!=0} then {set-footer $m; return}
    click-redisplay
    set-footer "latest save renamed as $savefile"
    if {$CSRstarted == 1} then {
        record-current-save-file config $savefile
    }
    if {$SSRstarted == 1} then {
        record-current-save-file setup $savefile
    }
}

proc click-filename {name} {
   global directory CSRstarted SSRstarted
   if {![file exists [file join $directory latest]]} then \
      {set-footer "there's no latest save-set!!!"; return}
   set savefile [file join $directory $name]
   if {[file exists $savefile]} then \
      {set-footer "File $savefile already exists!!!"; return}
   set z [catch {:file rename [file join $directory latest] $savefile} m]
   if {$z!=0} then {set-footer $m; return}
   click-redisplay
   set-footer "latest save renamed as $savefile"
    if {$CSRstarted == 1} then {
        record-current-save-file config $savefile
    }
    if {$SSRstarted == 1} then {
        record-current-save-file setup $savefile
    }
}

proc click-delete {} {
   global directorylist selection directory
   if {[llength $selection]==0} then {
      set-footer "no file selected!!!"
      return
   }
   if {[llength $selection]>1} then {
      set-footer "multiple files selected!!!"
      return
   }
   set target [lindex $directorylist $selection]
   if {[string range $target 0 0] == "d"} then {
      set-footer "can't delete a directory!!!"
      return
   }
   set file [last $target]
   if {$file == "."} then {set-footer "can't delete `.'"; return}
   if {$file == ".."} then {set-footer "can't delete `.'"; return}
   clear-footer
   if {[midas-confirmation "Are you sure you want to delete $file?"] == 0} then {return}
   set-footer "deleting $file..."

   set z [catch {:file delete [file join $directory $file]} m]
   if {$z!=0} then {set-footer $m; return}
   click-redisplay
   set-footer "$file deleted"
}

proc click-SSRrestore {} {
   global directorylist selection directory
   if {[llength $selection]==0} then {
      set-footer "no save-file selected!!!"
      return
   }
   if {[llength $selection]>1} then {
      set-footer "multiple save-files selected!!!"
      return
   }
   set target [lindex $directorylist $selection]
   if {[string range $target 0 0] == "d"} then {
      set-footer "can't restore a directory!!!"
      return
   }
   set file [last $target]
   if {$file == "."} then {set-footer "can't restore `.'"; return}
   if {$file == ".."} then {set-footer "can't restore `.'"; return}

   restoreVMEregisters  [file join $directory $file]
}

proc click-SSRrestore-current {} {

    set f [recover-current-save-file setup]
    if {[file isfile $f] == 0} then {
        SSRstart
    } else {
        restoreVMEregisters $f
    }
}

proc click-CSRrestore {} {
   global directorylist selection directory
   if {[llength $selection]==0} then {
      set-footer "no save-file selected!!!"
      return
   }
   if {[llength $selection]>1} then {
      set-footer "multiple save-files selected!!!"
      return
   }
   set target [lindex $directorylist $selection]
   if {[string range $target 0 0] == "d"} then {
      set-footer "can't restore a directory!!!"
      return
   }
   set file [last $target]
   if {$file == "."} then {set-footer "can't restore `.'"; return}
   if {$file == ".."} then {set-footer "can't restore `.'"; return}

   restoreVMEconfiguration [file join $directory $file]
}

proc click-CSRrestore-current {} {

    set f [recover-current-save-file config]
    if {[file isfile $f] == 0} then {
        CSRstart
    } else {
        restoreVMEconfiguration $f
    }
}

proc restoreVMEconfiguration {file} {

   set-footer "restoring from $file..."
   set f [open $file r]
   set configuration [split [read $f] \n]
   close $f
   set l [expr [llength $configuration] - 1]
   set configuration [lreplace $configuration $l $l]
   send vmecontrol RestoreConfiguration [list $configuration]
   set-footer "restored configuration from $file"
   record-current-save-file config $file
}

proc restoreVMEregisters {file} {
   global debug RegisterServer EGreport

   set-footer "restoring from $file..."

   set f [open $file r]
   set l [read $f]
   close $f

   set EGreport_save $EGreport
   set EGreport -1
   set-busy
   set previouscard ""

   foreach item $l {
      if {$debug} then {puts stdout "restore: $item"}
      set register /$RegisterServer/[first $item]
      set value [second $item]
      if {$value == ""} then {continue}
      set card [first [split [first $item] .]]
      if {$card != $previouscard} then {
         set-footer   "restoring parameters to $card"
         set previouscard $card
      }
      if {$debug} then {puts stdout  "restoring parameter $register to value $value"}
      set z [catch {EG "register :name '$register' :value '$value'" } list]
   }
   set EGreport $EGreport_save
   clear-busy

   set-footer "restored module setup from $file"
   insert-log "VME [date] restored module setup from $file"

   record-current-save-file setup $file

   write-registers NetVar_HasBeenRestored 1
}

proc HasBeenRestored {} {
    set response [read-registers NetVar_HasBeenRestored]
    set l [llength $response]
    set vl 0
    foreach item $response {incr vl [second $item]}
    return [expr $vl/$l]
}

proc click-selective-restore {} {
   global directorylist selection directory cardtype lowmodule highmodule
   if {[llength $selection]==0} then {
      set-footer "no save-file selected!!!"
      return
   }
   if {[llength $selection]>1} then {
      set-footer "multiple save-files selected!!!"
      return
   }
   set target [lindex $directorylist $selection]
   if {[string range $target 0 0] == "d"} then {
      set-footer "can't restore a directory!!!"
      return
   }
   set file [last $target]
   if {$file == "."} then {set-footer "can't restore `.'"; return}
   if {$file == ".."} then {set-footer "can't restore `.'"; return}
   set pattern "[set cardtype]#\[$lowmodule-$highmodule\].*"
   selective-restore  [file join $directory $file] $pattern
   record-current-save-file setup  [file join $directory $file]
}

proc selective-restore {file pattern} {
   global debug RegisterServer EGreport

   set-footer "restoring parameters $pattern from $file..."

   set f [open $file r]
   set l [read $f]
   close $f
 
   set EGreport_save $EGreport
   set EGreport -1
   set-busy

   foreach item $l {
#      puts stdout " $pattern $item "
      if {[match-name $pattern [first $item]]} then {
         set register /$RegisterServer/[first $item]
         set value [second $item]
          set-footer   "restoring parameter $register to value $value"
          if {$debug} then {puts stdout  "restoring parameter $register to value $value"}
         set z [catch {EG "register :name '$register' :value '$value'" } list]
      }
   }
   set EGreport $EGreport_save
   clear-busy

   set-footer "selective restore of setup from $file"
}


proc click arg {
   global debug directory selection SSRstarted
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch $w {
      DIRECTORY       {click-directory $v}
      DIRECTORYLIST   {click-list $v $i}
      UP              {click-up}
      DOWN            {click-down}
      LOW             {click-low $v}
      HIGH            {click-high $v}
      CARD            {click-card $v}
      SAVE            {if {$SSRstarted == 1} then {click-SSRsave} else {click-CSRsave}}
      TIME-STAMP      {click-time-stamp}
      FILENAME        {click-filename $v}
      DELETE          {click-delete}
      RESTORE         {if {$SSRstarted == 1} then {click-SSRrestore} else {click-CSRrestore}}
      RESTORE-CURRENT {if {$SSRstarted == 1} then {click-SSRrestore-current} else {click-CSRrestore-current}}
      SELECTIVE-RESTORE {click-selective-restore}
      REDISPLAY       {click-redisplay}
      SHOW            {click-show}
      HELP            {click-help}
      default         {click-on $w $v}
      }
   }


proc start args {SSRstart}
proc SSRstart args {
   global env SSRstarted CSRstarted moduledefinition directory cardtypes cardtype RegisterServer

   if {$SSRstarted != 0} then {open-frame} else {
     set CSRstarted 0
     set SSRstarted 1

     make-SSRframe

     inform vmecontrol autostart
     set RegisterServer [inform vmecontrol inquire Role]

     set z [catch {set directory [file join $env(EXPERIMENTBASE) VMEsetups]} m]
     if {$z != 0} then {
        midas-warning "$m\nYou should probably Join an experiment!"
        close-frame
        set SSRstarted 0
        return
     }

     if {![file exists $directory]} then {
        set z [catch {:file mkdir $directory} m]
        if {$z!=0} then {
           midas-warning "Cannot create directory $directory\n $m"
           close-frame
           set SSRstarted 0
           return
        }
     }    
 
     set moduledefinition [inform vmecontrol inquire moduledefinition]

     set cardtypes {}
     foreach item $moduledefinition {
        set moduletype [first $item]
        if {$moduletype != ""} then {lappend cardtypes [second $item]}
     }
     set cardtype [first $cardtypes]

     :modify CARD :strings $cardtypes
     click-redisplay
   }
}

proc CSRstart args {
   global env SSRstarted CSRstarted directory RegisterServer

   if {$CSRstarted != 0} then {open-frame} else {
     set SSRstarted 0
     set CSRstarted 1

     make-CSRframe

     inform vmecontrol autostart
     set RegisterServer [inform vmecontrol inquire Role]

     set z [catch {set directory [file join $env(EXPERIMENTBASE) VMEconfigs]} m]
     if {$z != 0} then {
        midas-warning "$m\nYou should probably Join an experiment!"
        close-frame
        return
     }

     if {![file exists $directory]} then {
        set z [catch {:file mkdir $directory} m]
        if {$z!=0} then {midas-warning "Cannot create directory $directory\n $m"}
#        :file chmod 777 $directoryopen
     }    
 
     click-redisplay
   }
}

proc click-quit args {close-frame}
proc finish args {destroy-frame}


proc make-SSRframe {} {
  global env frametitle cardtypes

   set frametitle "Saving and Restoring VME Module Setups"

    :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

      :panel P1 :layout T :padx 10 :pady 10
         :layout H; :row-gap 0; :col-gap 10

         :text DIRECTORY :label "Directory:" :w 60 :shifters T
         :button UP 
         :button DOWN

      :panel P2 :layout T :padx 10
         :list DIRECTORYLIST :exclusive T :rows 10 :cols 80

      :panel P3 :layout T :padx 10 :pady 10
         :layout H
        :col-gap 10; :row-gap 10
         :number LOW  :label "Low module:"  :min 0 :max 99 :value 1 :w 4 :valuex 12c
         :number HIGH :label "High module:" :min 0 :max 99 :value 1 :w 4 :valuex 12c
         :choice-stack CARD :label "Card type:" :exclusive T :strings $cardtypes :value 0
         :hskip 60
         :button SELECTIVE-RESTORE :label "Selective Restore"
         :next-row T
         :button RESTORE
         :button SAVE
         :button TIME-STAMP      :label "Time Stamp"
         :text FILENAME :label Filename: :w 24 :value ""
         :button RESTORE-CURRENT :label "Restore Current"
        :next-row T
         :button REDISPLAY
         :button DELETE
         :button SHOW
         :hskip 100
         :button HELP       :label Help... 
      
      :show T
}



proc make-CSRframe {} {
  global env frametitle

   set frametitle "Saving and Restoring VME Module Configurations"

    :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

      :panel P1 :layout T :padx 10 :pady 10
         :layout H; :row-gap 0; :col-gap 10

         :text DIRECTORY :label "Directory:" :w 60  :shifters T
         :button UP
         :button DOWN

      :panel P2 :layout T :padx 10
         :list DIRECTORYLIST :exclusive T :rows 10 :cols 80

      :panel P3 :layout T :padx 10 :pady 10
         :layout H
        :col-gap 10; :row-gap 10
         :button RESTORE
         :button SAVE
         :button TIME-STAMP      :label "Time Stamp"
         :text FILENAME :label Filename: :w 24 :value ""
         :button RESTORE-CURRENT :label "Restore Current"
        :next-row T
         :button REDISPLAY
         :button DELETE
         :button SHOW
         :hskip 100
         :button HELP       :label Help...
      
      :show T
}
